@@ -1,6 +1,7 @@
 from typing import Optional
 
 import qrcode
+import qrcode.exceptions
 
 from PyQt5.QtGui import QColor, QPen
 import PyQt5.QtGui as QtGui
@@ -16,28 +17,40 @@ from electrum.simple_config import SimpleConfig
 from .util import WindowModalDialog, WWLabel, getSaveFileName
 
 
+class QrCodeDataOverflow(qrcode.exceptions.DataOverflowError):
+    pass
+
+
 class QRCodeWidget(QWidget):
 
-    def __init__(self, data = None):
+    def __init__(self, data=None, *, manual_size: bool = False):
         QWidget.__init__(self)
         self.data = None
         self.qr = None
         self._framesize = None  # type: Optional[int]
+        self._manual_size = manual_size
         self.setData(data)
 
-
     def setData(self, data):
-        if self.data != data:
-            self.data = data
-        if self.data:
-            self.qr = qrcode.QRCode(
+        if data:
+            qr = qrcode.QRCode(
                 error_correction=qrcode.constants.ERROR_CORRECT_L,
                 box_size=10,
                 border=0,
             )
-            self.qr.add_data(self.data)
+            try:
+                qr.add_data(data)
+                qr_matrix = qr.get_matrix()  # test that data fits in QR code
+            except (ValueError, qrcode.exceptions.DataOverflowError) as e:
+                raise QrCodeDataOverflow() from e
+            self.qr = qr
+            self.data = data
+            if not self._manual_size:
+                k = len(qr_matrix)
+                self.setMinimumSize(k * 5, k * 5)
         else:
             self.qr = None
+            self.data = None
 
         self.update()
 
@@ -47,8 +60,9 @@ class QRCodeWidget(QWidget):
             return
 
         black = QColor(0, 0, 0, 255)
+        grey  = QColor(196, 196, 196, 255)
         white = QColor(255, 255, 255, 255)
-        black_pen = QPen(black)
+        black_pen = QPen(black) if self.isEnabled() else QPen(grey)
         black_pen.setJoinStyle(Qt.MiterJoin)
 
         if not self.qr:
@@ -82,13 +96,14 @@ class QRCodeWidget(QWidget):
         qp.setPen(white)
         qp.drawRect(0, 0, framesize, framesize)
         # Draw qr code
-        qp.setBrush(black)
+        qp.setBrush(black if self.isEnabled() else grey)
         qp.setPen(black_pen)
         for r in range(k):
             for c in range(k):
                 if matrix[r][c]:
-                    qp.drawRect(int(left+c*boxsize), int(top+r*boxsize),
-                                boxsize - 1, boxsize - 1)
+                    qp.drawRect(
+                        int(left+c*boxsize), int(top+r*boxsize),
+                        boxsize - 1, boxsize - 1)
         qp.end()
 
     def grab(self) -> QtGui.QPixmap:
@@ -120,7 +135,7 @@ class QRDialog(WindowModalDialog):
 
         vbox = QVBoxLayout()
 
-        qrw = QRCodeWidget(data)
+        qrw = QRCodeWidget(data, manual_size=True)
         qrw.setMinimumSize(250, 250)
         vbox.addWidget(qrw, 1)
 
