@@ -54,9 +54,14 @@ class StorageEncryptionVersion(IntEnum):
 class StorageReadWriteError(Exception): pass
 
 
+class StorageOnDiskUnexpectedlyChanged(Exception): pass
+
+
 # TODO: Rename to Storage
 class WalletStorage(Logger):
 
+    # TODO maybe split this into separate create() and open() classmethods, to prevent some bugs.
+    #      Until then, the onus is on the caller to check file_exists().
     def __init__(self, path):
         Logger.__init__(self)
         self.path = standardize_path(path)
@@ -69,12 +74,16 @@ class WalletStorage(Logger):
         except IOError as e:
             raise StorageReadWriteError(e) from e
         if self.file_exists():
-            with open(self.path, "r", encoding='utf-8') as f:
-                self.raw = f.read()
+            with open(self.path, "rb") as f:
+                self.raw = f.read().decode("utf-8")
+                self.pos = f.seek(0, os.SEEK_END)
+                self.init_pos = self.pos
             self._encryption_version = self._init_encryption_version()
         else:
             self.raw = ''
             self._encryption_version = StorageEncryptionVersion.PLAINTEXT
+            self.pos = 0
+            self.init_pos = 0
 
     def read(self):
         return self.decrypted if self.is_encrypted() else self.raw
@@ -82,16 +91,15 @@ class WalletStorage(Logger):
     def write(self, data: str) -> None:
         s = self.encrypt_before_writing(data)
         temp_path = "%s.tmp.%s" % (self.path, os.getpid())
-        with open(temp_path, "w", encoding='utf-8') as f:
-            f.write(s)
+        with open(temp_path, "wb") as f:
+            f.write(s.encode("utf-8"))
+            self.pos = f.seek(0, os.SEEK_END)
             f.flush()
             os.fsync(f.fileno())
-
         try:
             mode = os.stat(self.path).st_mode
         except FileNotFoundError:
             mode = stat.S_IREAD | stat.S_IWRITE
-
         # assert that wallet file does not exist, to prevent wallet corruption (see issue #5082)
         if not self.file_exists():
             assert not os.path.exists(self.path)
@@ -100,6 +108,21 @@ class WalletStorage(Logger):
         self._file_exists = True
         self.logger.info(f"saved {self.path}")
 
+    def append(self, data: str) -> None:
+        """ append data to file. for the moment, only non-encrypted file"""
+        assert not self.is_encrypted()
+        with open(self.path, "rb+") as f:
+            pos = f.seek(0, os.SEEK_END)
+            if pos != self.pos:
+                raise StorageOnDiskUnexpectedlyChanged(f"expected size {self.pos}, found {pos}")
+            f.write(data.encode("utf-8"))
+            self.pos = f.seek(0, os.SEEK_END)
+            f.flush()
+            os.fsync(f.fileno())
+
+    def needs_consolidation(self):
+        return self.pos > 2 * self.init_pos
+
     def file_exists(self) -> bool:
         return self._file_exists
 
@@ -179,6 +202,7 @@ class WalletStorage(Logger):
     def encrypt_before_writing(self, plaintext: str) -> str:
         s = plaintext
         if self.pubkey:
+            self.decrypted = plaintext
             s = bytes(s, 'utf8')
             c = zlib.compress(s, level=zlib.Z_BEST_SPEED)
             enc_magic = self._get_encryption_magic()
